/**
 * 
 */
package org.bonitasoft.connector;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.logging.Logger;

import org.apache.pdfbox.exceptions.COSVisitorException;
import org.bonitasoft.custom.MergePdf;
import org.bonitasoft.engine.api.ProcessAPI;
import org.bonitasoft.engine.bpm.document.Document;
import org.bonitasoft.engine.bpm.document.DocumentNotFoundException;
import org.bonitasoft.engine.bpm.document.DocumentValue;
import org.bonitasoft.engine.bpm.document.impl.DocumentImpl;
import org.bonitasoft.engine.connector.ConnectorException;

/**
 *The connector execution will follow the steps
 * 1 - setInputParameters() --> the connector receives input parameters values
 * 2 - validateInputParameters() --> the connector can validate input parameters values
 * 3 - connect() --> the connector can establish a connection to a remote server (if necessary)
 * 4 - executeBusinessLogic() --> execute the connector
 * 5 - getOutputParameters() --> output are retrieved from connector
 * 6 - disconnect() --> the connector can close connection to remote server (if any)
 */
public class BonitasoftConnectorMergePDFImpl extends
		AbstractBonitasoftConnectorMergePDFImpl {

	@Override
	protected void executeBusinessLogic() throws ConnectorException{
		//Get access to the connector input parameters
		//getListDocumentsToMerge();
		//getPdfName1();
		//getPdfName2();
		//getProcessInstanceId();
		//getLogs();
		
		//TODO execute your business logic here 
	
		//WARNING : Set the output of the connector execution. If outputs are not set, connector fails
		//setOutputDocument(outputDocument);
		
		//var
				Boolean trace = getLogs();
				
				//Init logger
				Logger logger = Logger.getLogger("org.bonitasoft");
				logger.info(this.getClass().getName() + " - execute method executeBusinessLogic.");
				
				//Init processAPI
		    	ProcessAPI myProcessAPI = apiAccessor.getProcessAPI();
				
				//Get access to the connector input parameters
				String pdfName1 = getPdfName1();
				String pdfName2 = getPdfName2();
				List<DocumentImpl> listDocs = getListDocumentsToMerge();
				Long processInstanceId = getProcessInstanceId();
				
				if(trace){
					logger.info("pdfName1 : " + pdfName1);
					logger.info("pdfName2 : " + pdfName2);
					for(int i=0; i<listDocs.size();i++){
						logger.info("pdfNameInList : " + listDocs.get(i).getName());
					}
					logger.info("processInstanceId : " + processInstanceId);
				}
				
				//Init value return
		        DocumentValue documentValue = new DocumentValue("empty");
		        
		        //Business Logic
		        try {
		        	Document myDocument = null;
		        	byte[] docContent = null;
		        	
		        	List<InputStream> myList = new ArrayList<InputStream>();
		           	if(pdfName1!=null && !pdfName1.equalsIgnoreCase("")){	            		        	
		           		try {
		           			myDocument = myProcessAPI.getLastDocument(processInstanceId, pdfName1);
		        	       	docContent = myProcessAPI.getDocumentContent(myDocument.getContentStorageId());
				        	InputStream pdf1 = new ByteArrayInputStream(docContent);
				        	myList.add(pdf1);
		           		} catch (DocumentNotFoundException e) {
								// TODO Bloc catch généré automatiquement
								logger.severe(this.getClass().getName() + " - Error DocumentNotFoundException : " + e.getMessage());
						}
		           	}
		        	
		           	if(pdfName2!=null && !pdfName2.equalsIgnoreCase("")){
			        	try {
							myDocument = myProcessAPI.getLastDocument(processInstanceId, pdfName2);
							docContent = myProcessAPI.getDocumentContent(myDocument.getContentStorageId());
				        	InputStream pdf2 = new ByteArrayInputStream(docContent);
				        	myList.add(pdf2);
						} catch (DocumentNotFoundException e) {
							// TODO Bloc catch généré automatiquement
							logger.severe(this.getClass().getName() + " - Error DocumentNotFoundException : " + e.getMessage());
						}	
		           	}
		        	
		        	for(int i=0; i<listDocs.size();i++){
		        		if (listDocs.get(i)!=null){// && listDocs.get(i).getContentStorageId()!=null){
		        			try {
		        			docContent = myProcessAPI.getDocumentContent(listDocs.get(i).getContentStorageId());
			        		InputStream pdfList = new ByteArrayInputStream(docContent);
			        		myList.add(pdfList);
		        			}
		        			catch (DocumentNotFoundException e) {
								// TODO Bloc catch généré automatiquement
								logger.severe(this.getClass().getName() + " - Error DocumentNotFoundException : " + e.getMessage());
							}
		        		}
		        	}
		    		
		    		
		            MergePdf myMergePdf = new MergePdf();
		        	
		        	OutputStream myReturn = myMergePdf.actionMerge(myList);
					
					SimpleDateFormat format = new SimpleDateFormat("dd-MM-yyyy-HH-mm-ss");
		            String now = format.format(Calendar.getInstance().getTime());
		            String strOut = "combined_"+now+".pdf";
		            
		            byte[] bytes = ((ByteArrayOutputStream) myReturn).toByteArray();
		            
		            documentValue = new DocumentValue(bytes, "application/pdf", strOut);
		            logger.info(this.getClass().getName() + " - Merged PDF create : " + strOut);
				} catch (COSVisitorException e) {
					logger.severe(this.getClass().getName() + " - Error COSVisitorException : " + e.getMessage());
				} catch (IOException e) {
					logger.severe(this.getClass().getName() + " - Error IOException : " + e.getMessage());
				} 
		        
		        //Return
				setOutputDocument(documentValue);
	
	 }

	@Override
	public void connect() throws ConnectorException{
		//[Optional] Open a connection to remote server
	
	}

	@Override
	public void disconnect() throws ConnectorException{
		//[Optional] Close connection to remote server
	
	}

}
